#!/usr/bin/env python

"""
Synopsis:
    Write report from Vcloud login response XML doc.
Usage:
    ./admin_report.py infile [outfile]
Examples:
    ./admin_report.py infile.xml outfile.txt
    ./admin_report.py infile.xml
"""

import sys
import os
import argparse
from lxml import objectify


class Writer(object):
    def __init__(self, outfile):
        self.outfile = outfile

    def write(self, content):
        self.outfile.write(content)

    def writeln(self, content):
        self.outfile.write(content)
        self.outfile.write('\n')


def modify(infile, outfile, options):
    pass


def report(infile, outfile, options):
    writer = Writer(outfile)
    wrtln = writer.writeln
    doc = objectify.parse(infile)
    root = doc.getroot()
    wrtln('Vcloud admin report')
    wrtln('===================')
    report_vcloud(root, wrtln, options)


def report_vcloud(root, wrtln, options):
    wrtln('Vcloud:')
    wrtln('  name: {}'.format(root.get('name')))
    wrtln('  href: {}'.format(root.get('href')))
    report_links(root, wrtln, options)
    if hasattr(root, 'Description'):
        report_description(root.Description, wrtln, options)
    if hasattr(root, 'OrganizationReferences'):
        report_organization_refs(root.OrganizationReferences, wrtln, options)
    if hasattr(root, 'ProviderVdcReferences'):
        report_provider_refs(root.ProviderVdcReferences, wrtln, options)
    if hasattr(root, 'RightReferences'):
        report_right_refs(root.RightReferences, wrtln, options)
    if hasattr(root, 'RoleReferences'):
        report_role_refs(root.RoleReferences, wrtln, options)
    if hasattr(root, 'Networks'):
        report_networks(root.Networks, wrtln, options)


def report_links(root, wrtln, options):
    for link in root.iterchildren(
            tag='{http://www.vmware.com/vcloud/v1.5}Link'):
        wrtln('  Link:')
        wrtln('    rel : {}'.format(link.get('rel', '<unknown>')))
        wrtln('    type: {}'.format(link.get('type', '<unknown>')))
        wrtln('    href: {}'.format(link.get('href', '<unknown>')))
        wrtln('    name: {}'.format(link.get('name', '<unknown>')))


def report_description(desc, wrtln, options):
    wrtln('  Description: "{}"'.format(desc.text.encode('utf-8')))


def report_organization_refs(refs, wrtln, options):
    for ref in refs.iterchildren(
            tag="{http://www.vmware.com/vcloud/v1.5}OrganizationReference"):
        wrtln('  Organization reference:')
        wrtln('    type: {}'.format(ref.get('type', '<unknown>')))
        wrtln('    href: {}'.format(ref.get('href', '<unknown>')))
        wrtln('    name: {}'.format(ref.get('name', '<unknown>')))


def report_provider_refs(refs, wrtln, options):
    for ref in refs.iterchildren(
            tag="{http://www.vmware.com/vcloud/v1.5}ProviderVdcReference"):
        wrtln('  Provider Vdc reference:')
        wrtln('    type: {}'.format(ref.get('type', '<unknown>')))
        wrtln('    href: {}'.format(ref.get('href', '<unknown>')))
        wrtln('    name: {}'.format(ref.get('name', '<unknown>')))


def report_right_refs(refs, wrtln, options):
    for ref in refs.iterchildren(
            tag="{http://www.vmware.com/vcloud/v1.5}RightReference"):
        wrtln('  Right reference:')
        wrtln('    type: {}'.format(ref.get('type', '<unknown>')))
        wrtln('    href: {}'.format(ref.get('href', '<unknown>')))
        wrtln('    name: {}'.format(ref.get('name', '<unknown>')))


def report_role_refs(refs, wrtln, options):
    for ref in refs.iterchildren(
            tag="{http://www.vmware.com/vcloud/v1.5}RoleReference"):
        wrtln('  Role reference:')
        wrtln('    type: {}'.format(ref.get('type', '<unknown>')))
        wrtln('    href: {}'.format(ref.get('href', '<unknown>')))
        wrtln('    name: {}'.format(ref.get('name', '<unknown>')))


def report_networks(refs, wrtln, options):
    for ref in refs.iterchildren(
            tag="{http://www.vmware.com/vcloud/v1.5}Network"):
        wrtln('  Network:')
        wrtln('    type: {}'.format(ref.get('type', '<unknown>')))
        wrtln('    href: {}'.format(ref.get('href', '<unknown>')))
        wrtln('    name: {}'.format(ref.get('name', '<unknown>')))


def main():
    parser = argparse.ArgumentParser(
        description=__doc__,
        formatter_class=argparse.RawDescriptionHelpFormatter,)
    parser.add_argument(
        'command', type=str,
        choices=['report', 'modify', ],
        help='command')
    parser.add_argument(
        'infile', type=str,
        help='name of input file to read from')
    parser.add_argument(
        'outfile', type=str, nargs='?', default=None,
        help='name of output file to write to.  If omitted: sys.stdout')
    parser.add_argument(
        '-f', '--force', action='store_true', default=False,
        help='force overwrite of output file')
    options = parser.parse_args()
    if not os.path.exists(options.infile):
        parser.error('input file ({}) does not exist'.format(options.infile))
    else:
        infile = open(options.infile, 'r')
    if options.outfile is None:
        outfile = sys.stdout
    else:
        if not options.force and os.path.exists(options.outfile):
            parser.error('out file ({}) exists; use -f to overwrite'.format(
                options.outfile))
        else:
            outfile = open(options.outfile, 'w')
    if options.command == 'report':
        report(infile, outfile, options)
    elif options.command == 'modify':
        modify(infile, outfile, options)


if __name__ == '__main__':
    #import ipdb; ipdb.set_trace()
    main()
