#!/usr/bin/env python

"""
Synopsis:
    Write report from Vcloud login response XML doc.
Usage:
    ./link_report.py infile [outfile]
Examples:
    ./link_report.py infile.xml outfile.txt
    ./link_report.py infile.xml
"""

import sys
import os
import argparse
from lxml import objectify


class Writer(object):
    def __init__(self, outfile):
        self.outfile = outfile

    def write(self, content):
        self.outfile.write(content)

    def writeln(self, content):
        self.outfile.write(content)
        self.outfile.write('\n')


def report(infile, outfile, options):
    writer = Writer(outfile)
    wrtln = writer.writeln
    doc = objectify.parse(infile)
    root = doc.getroot()
    wrtln('Login response report')
    report_session(root, wrtln, options)


def report_session(root, wrtln, options):
    wrtln('Session:')
    wrtln('--------')
    wrtln('  User: {}'.format(root.get('user')))
    wrtln('  Org : {}'.format(root.get('org')))
    report_links(root, wrtln, options)


def report_links(root, wrtln, options):
    for link in root.iterchildren(
            tag='{http://www.vmware.com/vcloud/v1.5}Link'):
        wrtln('  Link:')
        wrtln('    rel : {}'.format(link.get('rel')))
        wrtln('    type: {}'.format(link.get('type')))
        wrtln('    href: {}'.format(link.get('href')))


def main():
    parser = argparse.ArgumentParser(
        description=__doc__,
        formatter_class=argparse.RawDescriptionHelpFormatter,)
    parser.add_argument(
        'infile', type=str,
        help='name of input file to read from')
    parser.add_argument(
        'outfile', type=str, nargs='?', default=None,
        help='name of output file to write to.  If omitted: sys.stdout')
    parser.add_argument(
        '-f', '--force', action='store_true', default=False,
        help='force overwrite of output file')
    options = parser.parse_args()
    if not os.path.exists(options.infile):
        parser.error('input file ({}) does not exist'.format(options.infile))
    else:
        infile = open(options.infile, 'r')
    if options.outfile is None:
        outfile = sys.stdout
    else:
        if not options.force and os.path.exists(options.outfile):
            parser.error('out file ({}) exists; use -f to overwrite'.format(
                options.outfile))
        else:
            outfile = open(options.outfile, 'w')
    report(infile, outfile, options)


if __name__ == '__main__':
    #import ipdb; ipdb.set_trace()
    main()
